@@ -10,6 +10,7 @@ Alarms and Time zone objects does not have any class as for now.  Those are typi
 Users of the library should not need to construct any of those objects.  To add new content to the calendar, use ``calendar.save_event``, ``calendar.save_todo`` or ``calendar.save_journal``.  Those methods will return a CalendarObjectResource.
 """
 import logging
+import re
 import sys
 import uuid
 import warnings
@@ -730,6 +731,7 @@ class CalendarObjectResource(DAVObject):
         if not path and getattr(self, "path", None):
             path = self.path
         if id is None and path is not None and str(path).endswith(".ics"):
+            ## TODO: do we ever get here?  Perhaps this if is completely moot?
             id = re.search("(/|^)([^/]*).ics", str(path)).group(2)
         if id is None:
             id = str(uuid.uuid1())
@@ -1009,8 +1011,10 @@ class CalendarObjectResource(DAVObject):
         potentially breaking couplings
         """
         return (
-            self._data or self._vobject_instance or self._icalendar_instance
-        ) and self.data.count("BEGIN:") > 1
+            (self._data and self._data.count("BEGIN:") > 1)
+            or self._vobject_instance
+            or self._icalendar_instance
+        )
 
     def has_component(self):
         """
@@ -1145,6 +1149,18 @@ class CalendarObjectResource(DAVObject):
     )
 
     def _set_icalendar_instance(self, inst):
+        if not isinstance(inst, icalendar.Calendar):
+            ## assume inst is an Event, Journal or Todo.
+            ## TODO: perhaps a bit better sanity checking here?
+            try:  ## DEPRECATION TODO: remove this try/except the future
+                ## icalendar 7.x behaviour (not released yet as of 2025-09
+                cal = icalendar.Calendar.new()
+            except:
+                cal = icalendar.Calendar()
+                cal.add("prodid", "-//python-caldav//caldav//en_DK")
+                cal.add("version", "2.0")
+            cal.add_component(inst)
+            inst = cal
         self._icalendar_instance = inst
         self._data = None
         self._vobject_instance = None
