@@ -14,18 +14,27 @@ from typing import TYPE_CHECKING
 from typing import Union
 from urllib.parse import unquote
 
-import requests
+
+try:
+    import niquests as requests
+    from niquests.auth import AuthBase
+    from niquests.models import Response
+    from niquests.structures import CaseInsensitiveDict
+except ImportError:
+    import requests
+    from requests.auth import AuthBase
+    from requests.models import Response
+    from requests.structures import CaseInsensitiveDict
+
 from lxml import etree
 from lxml.etree import _Element
-from requests.auth import AuthBase
-from requests.models import Response
-from requests.structures import CaseInsensitiveDict
 
 from .elements.base import BaseElement
 from caldav import __version__
 from caldav.collection import Calendar
 from caldav.collection import CalendarSet
 from caldav.collection import Principal
+from caldav.compatibility_hints import FeatureSet
 from caldav.elements import cdav
 from caldav.elements import dav
 from caldav.lib import error
@@ -434,7 +443,7 @@ class DAVResponse:
 
 class DAVClient:
     """
-    Basic client for webdav, uses the requests lib; gives access to
+    Basic client for webdav, uses the niquests lib; gives access to
     low-level operations towards the caldav server.
 
     Unless you have special needs, you should probably care most about
@@ -458,6 +467,7 @@ class DAVClient:
         ssl_cert: Union[str, Tuple[str, str], None] = None,
         headers: Mapping[str, str] = None,
         huge_tree: bool = False,
+        features: Union[FeatureSet, dict] = None,
     ) -> None:
         """
         Sets up a HTTPConnection object towards the server in the url.
@@ -466,11 +476,12 @@ class DAVClient:
           url: A fully qualified url: `scheme://user:pass@hostname:port`
           proxy: A string defining a proxy server: `scheme://hostname:port`. Scheme defaults to http, port defaults to 8080.
           auth: A niquests.auth.AuthBase or requests.auth.AuthBase object, may be passed instead of username/password.  username and password should be passed as arguments or in the URL
-          timeout and ssl_verify_cert are passed to requests.request.
+          timeout and ssl_verify_cert are passed to niquests.request.
           if auth_type is given, the auth-object will be auto-created. Auth_type can be ``bearer``, ``digest`` or ``basic``. Things are likely to work without ``auth_type`` set, but if nothing else the number of requests to the server will be reduced, and some servers may require this to squelch warnings of unexpected HTML delivered from the
            server etc.
           ssl_verify_cert can be the path of a CA-bundle or False.
           huge_tree: boolean, enable XMLParser huge_tree to handle big events, beware of security issues, see : https://lxml.de/api/lxml.etree.XMLParser-class.html
+          features: The default, None, will in version 2.x enable all existing workarounds in the code for backward compability.  Otherwise it will expect a FeatureSet or a dict as defined in `caldav.compatibility_hints` and use that to figure out what workarounds are needed.
 
         The niquests library will honor a .netrc-file, if such a file exists
         username and password may be omitted.
@@ -481,22 +492,24 @@ class DAVClient:
         If the caldav server is behind a proxy or replies with html instead of xml
         when returning 401, warnings will be printed which might be unwanted.
         Check auth parameter for details.
-
-        TODO: we switched back from niquest to request in 2.0.1 due to https://github.com/python-caldav/caldav/issues/530, the text above may not me correct anymore and there are known bugs with the requests library.
         """
         headers = headers or {}
 
         ## Deprecation TODO: give a warning, user should use get_davclient or auto_calendar instead
 
-        self.session = requests.Session()
+        try:
+            self.session = requests.Session(multiplexed=True)
+        except TypeError:
+            self.session = requests.Session()
 
         log.debug("url: " + str(url))
         self.url = URL.objectify(url)
         self.huge_tree = huge_tree
+        self.features = FeatureSet(features)
         # Prepare proxy info
         if proxy is not None:
             _proxy = proxy
-            # requests library expects the proxy url to have a scheme
+            # niquests library expects the proxy url to have a scheme
             if "://" not in proxy:
                 _proxy = self.url.scheme + "://" + proxy
 
@@ -581,8 +594,6 @@ class DAVClient:
         """
         Instead of returning the current logged-in principal, it attempts to query for all principals. This may or may not work dependent on the permissions and implementation of the calendar server.
         """
-        ## TODO: allow server side filtering.  We need a  <D:property-search><D:prop><D:displayname/></D:prop><D:match>{name}</D:match></D:property-search> inside the PrincipalPropertySearch
-
         if name:
             name_filter = [
                 dav.PropertySearch()
@@ -595,13 +606,15 @@ class DAVClient:
         query = (
             dav.PrincipalPropertySearch()
             + name_filter
-            + [dav.Prop() + cdav.CalendarHomeSet() + dav.DisplayName()]
+            + [dav.Prop(), cdav.CalendarHomeSet(), dav.DisplayName()]
         )
         response = self.report(self.url, etree.tostring(query.xmlelement()))
         principal_dict = response.find_objects_and_props()
         ret = []
         for x in principal_dict:
             p = principal_dict[x]
+            if not dav.DisplayName.tag in p:
+                continue
             name = p[dav.DisplayName.tag].text
             error.assert_(not p[dav.DisplayName.tag].getchildren())
             error.assert_(not p[dav.DisplayName.tag].items())
@@ -836,12 +849,12 @@ class DAVClient:
                     reason="Server provides bearer auth, but no password given.  The bearer token should be configured as password"
                 )
 
-            if auth_type == "digest":
-                self.auth = requests.auth.HTTPDigestAuth(self.username, self.password)
-            elif auth_type == "basic":
-                self.auth = requests.auth.HTTPBasicAuth(self.username, self.password)
-            elif auth_type == "bearer":
-                self.auth = HTTPBearerAuth(self.password)
+        if auth_type == "digest":
+            self.auth = requests.auth.HTTPDigestAuth(self.username, self.password)
+        elif auth_type == "basic":
+            self.auth = requests.auth.HTTPBasicAuth(self.username, self.password)
+        elif auth_type == "bearer":
+            self.auth = HTTPBearerAuth(self.password)
 
     def request(
         self,
@@ -973,17 +986,6 @@ class DAVClient:
             self.password = None
             return self.request(str(url_obj), method, body, headers)
 
-        # this is an error condition that should be raised to the application
-        if (
-            response.status == requests.codes.forbidden
-            or response.status == requests.codes.unauthorized
-        ):
-            try:
-                reason = response.reason
-            except AttributeError:
-                reason = "None given"
-            raise error.AuthorizationError(url=str(url_obj), reason=reason)
-
         if error.debug_dump_communication:
             import datetime
             from tempfile import NamedTemporaryFile
@@ -1015,6 +1017,17 @@ class DAVClient:
                     commlog.write(to_wire(response._raw))
                 commlog.write(b"\n")
 
+        # this is an error condition that should be raised to the application
+        if (
+            response.status == requests.codes.forbidden
+            or response.status == requests.codes.unauthorized
+        ):
+            try:
+                reason = response.reason
+            except AttributeError:
+                reason = "None given"
+            raise error.AuthorizationError(url=str(url_obj), reason=reason)
+
         return response
 
 
@@ -1089,19 +1102,21 @@ def get_davclient(
         try:
             from conf import client
 
-            idx = None
-            if name:
+            idx = os.environ.get("PYTHON_CALDAV_TEST_SERVER_IDX")
+            try:
+                idx = int(idx)
+            except (ValueError, TypeError):
+                idx = None
+            name = name or os.environ.get("PYTHON_CALDAV_TEST_SERVER_NAME")
+            if name and not idx:
                 try:
                     idx = int(name)
                     name = None
                 except ValueError:
                     pass
-            try:
-                conn = client(idx, name)
-                if conn:
-                    return conn
-            except:
-                error.weirdness("traceback from client()")
+            conn = client(idx, name)
+            if conn:
+                return conn
         except ImportError:
             pass
         finally:
