@@ -93,10 +93,11 @@ def fix(event):
         global fixup_error_loggings
         fixup_error_loggings += 1
         is_power_of_two = lambda n: not (n & (n - 1))
+        logger = logging.getLogger("caldav")
         if is_power_of_two(fixup_error_loggings):
-            log = logging.warning
+            log = logger.warning
         else:
-            log = logging.debug
+            log = logger.debug
 
         log_message = [
             "Ical data was modified to avoid compatibility issues",
@@ -164,7 +165,10 @@ def create_ical(ical_fragment=None, objtype=None, language="en_DK", **props):
         my_instance = icalendar.Calendar()
         if objtype is None:
             objtype = "VEVENT"
-        component = icalendar.cal.component_factory[objtype]()
+        try:
+            component = icalendar.cal.component_factory[objtype]()
+        except:
+            component = icalendar.cal.component_factory.ComponentFactory()[objtype]()
         my_instance.add_component(component)
         ## STATUS should default to NEEDS-ACTION for tasks, if it's not set
         ## (otherwise we cannot easily add a task to a davical calendar and
